/* 							       	      HTAssoc.c
**	ASSOCIATION LIST FOR STORING NAME-VALUE PAIRS.
**
**	(c) COPYRIGHT MIT 1995.
**	Please first read the full copyright statement in the file COPYRIGH.
**	@(#) $Id: HTAssoc.c,v 2.13 1996/04/12 17:45:52 frystyk Exp $
**
**	NAMES NOT CASE SENSITIVE, AND ONLY COMMON LENGTH
**	IS CHECKED (allows abbreviations; well, length is
**	taken from lookup-up name, so if table contains
**	a shorter abbrev it is not found).
** AUTHORS:
**	AL	Ari Luotonen	luotonen@dxcern.cern.ch
**
** HISTORY:
**
**
** BUGS:
**
**
*/

/* Library include files */
#include "sysdep.h"
#include "HTUtils.h"
#include "HTString.h"
#include "HTAssoc.h"					 /* Implemented here */

PUBLIC HTAssocList *HTAssocList_new (void)
{
    return HTList_new();
}


PUBLIC BOOL HTAssocList_delete (HTAssocList * alist)
{
    if (alist) {
	HTAssocList *cur = alist;
	HTAssoc *assoc;
	while (NULL != (assoc = (HTAssoc*)HTList_nextObject(cur))) {
	    HT_FREE(assoc->name);
	    HT_FREE(assoc->value);
	    HT_FREE(assoc);
	}
	return HTList_delete(alist);
    }
    return NO;
}


PUBLIC BOOL HTAssocList_add (HTAssocList * alist,
			     const char * name, const char * value)
{
    HTAssoc *assoc;
    if (alist) {
	if ((assoc = (HTAssoc *) HT_CALLOC(1, sizeof(HTAssoc))) == NULL)
	    HT_OUTOFMEM("HTAssoc_add");
	if (name) StrAllocCopy(assoc->name, name);
	if (value) StrAllocCopy(assoc->value, value);
	return HTList_addObject(alist, (void *) assoc);
    } else {
	if (WWWTRACE)
	    HTTrace("HTAssoc_add: ERROR: assoc list NULL!!\n");
    }
    return NO;
}


PUBLIC char *HTAssocList_lookup (HTAssocList * alist, const char * name)
{
    HTAssocList *cur = alist;
    HTAssoc *assoc;
    while ((assoc = (HTAssoc *) HTList_nextObject(cur))) {
	if (!strncasecomp(assoc->name, name, strlen(name)))
	    return assoc->value;
    }
    return NULL;
}

